searchNodes=[{"doc":"Hammox is a library for rigorous unit testing using mocks, explicit behaviours and contract tests.See the README page for usage guide and examples.Most of the functions in this module come from Mox for backwards compatibility. As of v0.1.0, the only Hammox-specific functions are protect/2 and protect/3.","ref":"Hammox.html","title":"Hammox","type":"module"},{"doc":"See Mox.allow/3.","ref":"Hammox.html#allow/3","title":"Hammox.allow/3","type":"function"},{"doc":"See Mox.defmock/2.","ref":"Hammox.html#defmock/2","title":"Hammox.defmock/2","type":"function"},{"doc":"See Mox.expect/4.","ref":"Hammox.html#expect/4","title":"Hammox.expect/4","type":"function"},{"doc":"Takes the function provided by a module, function, arity tuple and decorates it with Hammox type checking.Returns a new anonymous function.Example:defmodule Calculator do @callback add(integer(), integer()) :: integer() end defmodule TestCalculator do def add(a, b), do: a + b end add_2 = Hammox.protect({TestCalculator, :add, 2}, Calculator) add_2.(1.5, 2.5) # throws Hammox.TypeMatchError","ref":"Hammox.html#protect/2","title":"Hammox.protect/2","type":"function"},{"doc":"Same as protect/2, but allows decorating multiple functions at the same time.Provide a list of functions to decorate as third argument.Returns a map where the keys are atoms of the form :{function_name}_{arity} and values are the decorated anonymous functions.Example:defmodule Calculator do @callback add(integer(), integer()) :: integer() @callback add(integer(), integer(), integer()) :: integer() @callback multiply(integer(), integer()) :: integer() end defmodule TestCalculator do def add(a, b), do: a + b def add(a, b, c), do: a + b + c def multiply(a, b), do: a * b end %{ add_2: add_2, add_3: add_3, multiply_2: multiply_2 } = Hammox.protect(TestCalculator, Calculator, add: [2, 3], multiply: 2)","ref":"Hammox.html#protect/3","title":"Hammox.protect/3","type":"function"},{"doc":"See Mox.set_mox_from_context/1.","ref":"Hammox.html#set_mox_from_context/1","title":"Hammox.set_mox_from_context/1","type":"function"},{"doc":"See Mox.set_mox_global/1.","ref":"Hammox.html#set_mox_global/1","title":"Hammox.set_mox_global/1","type":"function"},{"doc":"See Mox.set_mox_private/1.","ref":"Hammox.html#set_mox_private/1","title":"Hammox.set_mox_private/1","type":"function"},{"doc":"See Mox.stub/3.","ref":"Hammox.html#stub/3","title":"Hammox.stub/3","type":"function"},{"doc":"See Mox.stub_with/2.","ref":"Hammox.html#stub_with/2","title":"Hammox.stub_with/2","type":"function"},{"doc":"See Mox.verify!/0.","ref":"Hammox.html#verify!/0","title":"Hammox.verify!/0","type":"function"},{"doc":"See Mox.verify!/1.","ref":"Hammox.html#verify!/1","title":"Hammox.verify!/1","type":"function"},{"doc":"See Mox.verify_on_exit!/1.","ref":"Hammox.html#verify_on_exit!/1","title":"Hammox.verify_on_exit!/1","type":"function"},{"doc":"HammoxHex packageAPI docsHammox is a library for rigorous unit testing using mocks, explicit behaviours and contract tests. You can use it to ensure both your mocks and implementations fulfill the same contract.It takes the excellent Mox library and pushes its philosophy to its limits, providing automatic contract tests based on behaviour typespecs while maintaining full compability with code already using Mox.Hammox aims to catch as many contract bugs as possible while providing useful deep stacktraces so they can be easily tracked down and fixed.","ref":"readme.html","title":"Hammox","type":"extras"},{"doc":"If you are currently using Mox, delete it from your list of dependencies in mix.exs. Then add :hammox:def deps do [ {:hammox, &quot;~&gt; 0.2&quot;} ] end","ref":"readme.html#installation","title":"Hammox - Installation","type":"extras"},{"doc":"Read &quot;Mocks and explicit contracts&quot; by José Valim. Then proceed to the Mox documentation. Once you are comfortable with Mox, switch to using Hammox.","ref":"readme.html#starting-from-scratch","title":"Hammox - Starting from scratch","type":"extras"},{"doc":"Replace all occurences of Mox with Hammox. Nothing more is required; all your mock calls in test are now ensured to conform to the behaviour typespec.","ref":"readme.html#migrating-from-mox","title":"Hammox - Migrating from Mox","type":"extras"},{"doc":"Typical mock setupLet's say we have a database which can get us user data. We have a module, RealDatabase (not shown), which implements the following behaviour:defmodule Database do @callback get_users() :: [binary()] endWe use this client in a Stats module which can aggregate data about users:defmodule Stats do def count_users(database \\\\ RealDatabase) do length(database.get_users()) end endAnd we create a unit test for it:defmodule StatsTest do use ExUnit.Case, async: true test &quot;count_users/0 returns correct user count&quot; do assert 2 == Stats.count_users() end endFor this test to work, we would have to start a real instance of the database and provision it with two users. This is of course unnecessary brittleness — in a unit test, we only want to test that our Stats code provides correct results given specific inputs. To simplify, we will create a mocked Database using Mox and use it in the test:defmodule StatsTest do use ExUnit.Case, async: true import Mox test &quot;count_users/0 returns correct user count&quot; do defmock(DatabaseMock, for: Database) expect(DatabaseMock, :get_users, fn -&gt; [&quot;joe&quot;, &quot;jim&quot;] end) assert 2 == Stats.count_users(DatabaseMock) end endThe test now passes as expected.The contract breaksImagine that some time later we want to add error flagging for our database client. We change RealDatabase and the corresponding behaviour, Database, to return an ok/error tuple instead of a raw value:defmodule Database do @callback get_users() :: {:ok, [binary()]} | {:error, term()} endHowever, The Stats.count_users/0 test will still pass, even though the function will break when the real database client is used! This is because the mock is now invalid — it no longer implements the given behaviour, and therefore breaks the contract. Even though Mox is supposed to create mocks following explicit contracts, it does not take typespecs into account.This is where Hammox comes in. Simply swap Mox with Hammox and you will now get this when trying to run the test:** (Hammox.TypeMatchError) Returned value [&quot;joe&quot;, &quot;jim&quot;] does not match type {:ok, [binary()]} | {:error, term()}.Now the consistency between the mock and its behaviour is enforced.Completing the triangleHammox automatically checks mocks with behaviours, but what about the real implementations? The real goal is to keep all units implementing a given behaviour in sync.You can decorate any function with Hammox checks by using Hammox.protect/2. It will return an anonymous function which you can use in place of the original module function. An example test:defmodule RealDatabaseTest do use ExUnit.Case, async: true test &quot;get_users/0 returns list of users&quot; do get_users_0 = Hammox.protect({RealDatabase, :get_users, 0}, Database) assert {:ok, [&quot;jim&quot;, &quot;joe&quot;]} == get_users_0.() end endIt's a good idea to put setup logic like this in a setup_all hook and then access the protected functions using the test context:defmodule RealDatabaseTest do use ExUnit.Case, async: true setup_all do %{get_users_0: Hammox.protect({RealDatabase, :get_users, 0}, Database)} end test &quot;get_users/0 returns list of users&quot;, %{get_users_0: get_users_0} do assert {:ok, [&quot;jim&quot;, &quot;joe&quot;]} == get_users_0.() end endHammox also provides a setup_all friendly Hammox.protect/3 function which leverages this pattern. It produces a map of decorated functions from the module and is especially useful when you're decorating several functions at once:defmodule RealDatabaseTest do use ExUnit.Case, async: true setup_all do Hammox.protect(RealDatabase, Database, get_users: 0) end test &quot;get_users/0 returns list of users&quot;, %{get_users_0: get_users_0} do assert {:ok, [&quot;jim&quot;, &quot;joe&quot;]} == get_users_0.() end endWhy use Hammox for my application code when I have Dialyzer?Dialyzer cannot detect Mox style mocks not conforming to typespec.The main aim of Hammox is to enforce consistency between behaviours, mocks and implementations. This is best achieved when both mocks and implementations are subjected to the exact same checks.Dialyzer is a static analysis tool; Hammox is a dynamic contract test provider. They operate differently and one can catch some bugs when the other doesn't. While it is true that Hammox would be redudant given a strong, strict, TypeScript-like type system for Elixir, Dialyzer is far for providing that sort of coverage.","ref":"readme.html#example","title":"Hammox - Example","type":"extras"},{"doc":"A t() type defined on a protocol is taken by Hammox to mean &quot;a struct implementing the given protocol&quot;. Therefore, trying to pass :atom for an Enumerable.t() will produce an error, even though the type is defined as term():** (Hammox.TypeMatchError) Returned value :atom does not match type Enumerable.t(). Value :atom does not implement the Enumerable protocol.","ref":"readme.html#protocol-types","title":"Hammox - Protocol types","type":"extras"},{"doc":"Hammox also includes Mox as a dependency. This means that if you would like to disable Hammox protection for a specific mock, you can simply use vanilla Mox for that specific instance. They will interoperate without problems.","ref":"readme.html#disable-protection-for-specific-mocks","title":"Hammox - Disable protection for specific mocks","type":"extras"},{"doc":"For anonymous function types in typespecs, only the arity is checked. Parameter types and return types are not checked.","ref":"readme.html#limitations","title":"Hammox - Limitations","type":"extras"},{"doc":"Copyright 2019 Michał SzewczakLicensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License athttp://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#license","title":"Hammox - License","type":"extras"}]